import ply.lex, ply.yacc
import time


class CalcLexer(object):

    tokens = ('NUMBER',)

    t_ignore = " \t"

    literals = ['+','-','*','/','(',')']

    def t_NUMBER(self, t):
        r'\d+'
        try:
            t.value = int(t.value)
        except ValueError:
            print "Unable to convert to int", t.value
            t.value = 0
        return t

    def t_error(self, t):
        raise ValueError("Illegal Character '%s'" % t.value[0])

    def build(self, **kwargs):
        self.lexer = ply.lex.lex(object = self, **kwargs)

class CalcParser(object):
    tokens = CalcLexer.tokens

    precedence = (
            ('left', '+', '-'),
            ('left', '*', '/'),
            ('right', 'UMINUS')
            )

    def p_statement_expression(self, p):
        'statement : expression'
        p[0] = p[1]

    def p_expression_binop(self, p):
        """expression : expression '+' expression
                      | expression '-' expression
                      | expression '*' expression
                      | expression '/' expression"""

        if p[2] == '+':
            p[0] = p[1] + p[3]
        elif p[2] == '-':
            p[0] = p[1] - p[3]
        elif p[2] == '*':
            p[0] = p[1] * p[3]
        elif p[2] == '/':
           p[0] = p[1] / p[3]

    def p_expression_uminus(self, p):
        "expression : '-' expression %prec UMINUS"
        p[0] = -p[2]

    def p_expression_group(self, p):
        "expression : '(' expression ')'"

        p[0] = p[2]

    def p_expression_number(self, p):
        "expression : NUMBER"
        p[0] = p[1]

    def p_error(self, p):
        raise ValueError("Illegal Syntax '%s'" % p)


Lexer = CalcLexer()
Lexer.build()

ParserObj = CalcParser()
Parser = ply.yacc.yacc(module = ParserObj)

test = ["9 + 2 + 3",
      "9 + 2 *3"
      ]

iter = 1000
fStart = time.clock()

for j in range(iter):
    for t in test:
        Parser.parse(t)

fTime = time.clock() - fStart
print '%d iter, %f time, %f avg' % (iter, fTime, fTime/iter)


